package web.cps

import ant.cps.Form.{FieldOptions, FormFacade, FormProps, Item}
import google.maps.{LatLng, LatLngBounds}
import google.maps.places.{PlaceResult, SearchBox}
import web.base.data.PlaceFrag
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import org.scalajs.dom
import shared.models.Flds
import web.base.all._

import scala.scalajs.js
import scala.scalajs.js.Dynamic.literal
import scala.scalajs.js.annotation.ScalaJSDefined
import scala.scalajs.js.JSConverters._

object AddressInputMap {

  private final val MapHeight   = "200px"
  private final val DefaultIcon = "https://maps.gstatic.com/mapfiles/place_api/icons/geocode-71.png"

  class MarkerHelper(val icon: js.Dynamic, val title: String, val position: LatLng)

  object MarkerHelper {
    def apply(icon: String, title: String, position: LatLng): MarkerHelper =
      new MarkerHelper(newIcon(icon), title, position)

    def apply(title: String, position: LatLng): MarkerHelper = apply(DefaultIcon, title, position)
  }

  private def newIcon(icon: String) = {
    literal(
      url        = icon,
      size       = new google.maps.Size(71, 71),
      origin     = new google.maps.Point(0, 0),
      anchor     = new google.maps.Point(17, 34),
      scaledSize = new google.maps.Size(25, 25))
  }

  /**
    * Submit form is the only interesting one, basically on enter, things submit.  We take over enter and can
    * potentially
    * @param submitForm
    */
  case class Props(form: FormProps,
                   initialMarkers: Vector[MarkerHelper],
                   initialCenter: LatLng,
                   place: Option[PlaceFrag],
                   placeCallback: PlaceResult => CallbackTo[Unit],
                   submitForm: Option[Task[Unit]] = None)
      extends FormFacade

  case class State(markers: Vector[MarkerHelper], center: LatLng)

  class Backend(val $ : BackendScope[Props, State]) {

    var map: google.maps.Map = _
    var searchBox: SearchBox = _

    def mapMounted(ref: google.maps.Map)      = Callback { if (ref != null) map = ref }
    def searchMounted(ref: Option[SearchBox]) = Callback { ref.foreach(searchBox = _) }

    /**
      * The meat of hooking into google maps and the searchBox.  They
      */
    def placesChanged(places: Seq[PlaceResult]): Callback = {
      val bounds      = new LatLngBounds()
      var markers     = Vector.empty[MarkerHelper]
      val maybeCenter = places.headOption.map(_.geometry.location)

      places.collect {
        case place if place.geometry != null =>
          markers :+= new MarkerHelper(newIcon(place.icon), place.name, place.geometry.location)
          if (!js.isUndefined(place.geometry.viewport))
            bounds.union(place.geometry.viewport)
          else
            bounds.extend(place.geometry.location)
      }

      maybeCenter.fold(Callback.empty) { center =>
        // TODO: Should this be here?

        $.modState { s =>
          map.fitBounds(bounds)
          s.copy(markers, center)
        } >> $.props.flatMap(_.placeCallback(places.head))
      }
    }

    def render(p: Props, s: State) = {

      <.div(
        AddressInputSuggest.C(
          AddressInputSuggest.Props(
            changed       = placesChanged,
            searchMounted = searchMounted,
            defaultPlace  = p.place,
            form          = p.form,
            submitForm    = p.submitForm)),
        // Have to do this here because of the HOC embed
        Item(new Item.Opt {})(
          AddressMap(
            AddressMap.Props(
              boundsChange = Callback(searchBox.setBounds(map.getBounds())),
              mapMounted   = mapMounted,
              center       = s.center,
              markers      = s.markers),
            containerElement = <.div(^.height := MapHeight, ^.width := "100%"),
            mapElement       = <.div(^.height := MapHeight, ^.width := "100%"))))
    }
  }

  val C = ScalaComponent
    .builder[Props]("AddressInput")
    .initialStateCallbackFromProps[State](p => CallbackTo(State(p.initialMarkers, p.initialCenter)))
    .renderBackend[Backend]
    .build
}
