package web.cps

import ant.cps.Layout.Header
import ant.cps._
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.{EventListener, OnUnmount, Reusability}
import japgolly.scalajs.react.vdom.html_<^._
import org.scalajs.dom
import org.scalajs.dom.Event
import org.scalajs.dom.html.Element
import org.scalajs.dom.raw.HTMLElement
import web.base.all._
import web.base.data.JsCoerce

import scala.scalajs.js.JSConverters._
import scala.scalajs.js
import scala.scalajs.js.timers.SetIntervalHandle
import scala.scalajs.js.{|, UndefOr}

object AdjustingHeader {
  //
  val delta = 5

  case class State(didScroll: Boolean, navUp: Boolean)
  case class Props(height: Int, left: VdomElement, right: VdomElement, adjust: Boolean = true)

  val half = new Col.Opt {
    override val span = 12
  }

  implicit val stateReuse = Reusability.caseClass[State]
  implicit val vdomReuse  = Reusability.by_==[VdomElement]
  implicit val propsReuse = Reusability.caseClass[Props]

  class Backend(val $ : BackendScope[Props, State]) extends OnUnmount {

    def handleScroll(e: Event): CallbackTo[Unit] = $.modState(_.copy(true))
    var timer: js.UndefOr[SetIntervalHandle]     = js.undefined
    var lastScrollTop                            = 0.0

    def hasScrolled: Callback = $.props.flatMap { p =>
      //
      val st = dom.document.body.scrollTop
      val cb = if (Math.abs(lastScrollTop - st) <= delta) {
        //
        Callback.empty
      } else {
        if (st > lastScrollTop && st > p.height)
          $.modState(_.copy(navUp = true))
        else {
          if (st + dom.window.innerHeight < docHeight)
            $.modState(_.copy(navUp = false))
          else Callback.empty
        }
      }
      cb >> Callback(lastScrollTop = st)

    }

    def docHeight: Double = {
      val body = dom.document.body
      val html = dom.document.documentElement.asInstanceOf[HTMLElement]

      math.max(
        math.max(
          math.max(math.max(body.scrollHeight.toDouble, body.offsetHeight), html.clientHeight.toDouble),
          html.scrollHeight.toDouble),
        html.offsetHeight.toDouble)
    }

    def didMount = $.props.map { p =>
      if (p.adjust)
        timer = js.timers.setInterval(250) {
          $.state flatMap { s =>
            if (s.didScroll)
              hasScrolled >> $.modState(_.copy(false))
            else Callback.empty

          } runNow ()
        }
    }

    def willUnmount = Callback {
      timer foreach js.timers.clearInterval
      timer = js.undefined
    }

    def render(p: Props, pc: PropsChildren, s: State) = {
      <.div(
        Header
          .C(new Header.Opts {
            val className = "hdr-adjust"
          }.ex(TagMod.when(s.navUp)(^.top := "-" + p.height + "px")))(p.right, p.left),
        <.div(^.paddingTop := p.height + "px")(pc))
    }
  }

  val C = ScalaComponent
    .builder[Props]("AdjustingHeader")
    .initialState(State(false, false))
    .renderBackendWithChildren[Backend]
    .componentDidMount(_.backend.didMount)
    .componentWillUnmount(_.backend.willUnmount)
    .configure(
      EventListener[Event].install("scroll", _.backend.handleScroll, _ => dom.window),
      Reusability.shouldComponentUpdate)
    .build
}
