package web.cps

import ant.cps._
import ant.cps.Form._
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import web.base.all._

import scala.scalajs.js
import scala.scalajs.js.{|, UndefOr}

object DiscreteNumInput {
  def formatNum(i: Int): String    = { i.toString.reverse.sliding(3, 3).mkString(",").reverse }
  def formatUSD(i: Int): String    = "$" + formatNum(i)
  def formatUSD(i: Double): String = "$" + formatNum(i.toInt)

  case class Props(formProps: FormProps,
                   name: String,
                   min: Int,
                   max: Int,
                   options: Vector[Int],
                   default: Int,
                   tagMods: Seq[TagMod] = Nil,
                   fmt: Int => String   = _.toString)

  object Props {
    def apply(formProps: FormProps, name: String, orderedOptions: Vector[Int]): Props = {
      Props(formProps, name, orderedOptions(0), orderedOptions.last, orderedOptions, orderedOptions(0))
    }

    def apply(formProps: FormProps, name: String, orderedOptions: Vector[Int], fmt: Int => String): Props = {
      Props(formProps, name, orderedOptions(0), orderedOptions.last, orderedOptions, orderedOptions(0), fmt = fmt)
    }
  }

  /* Get rid of the non numeric characters. */
  def filterChars(s: String): String = { s.replaceAll("[^0-9]", "") }

  class Backend(val $ : BackendScope[Props, Unit]) extends FormFacade {

    def form: FormProps = $.props.runNow().formProps

    // ============================== Coverage Widget =====================
    /* Simple hook for the buttons, TODO: This could probably be thrown as it's own component */
    private def onIncDec(up: Boolean): ReactEvent => Callback = { ri =>
      ri.preventDefaultCB >> Callback {

        value.foreach { v =>
          val ud =
            if (up) v + 1
            else v - 1

          onChange(v, ud)
        }
      }
    }

    def mount: Callback = $.props.map { p =>
      val fieldValue = value.getOrElse(p.default)
      form.getFieldDecorator(p.name, new FormComponent.Opt {
        override val initialValue = fieldValue
      })
      ()
    }

    /** This handles dealing with bounds and steps.  It only allows for discrete values to be used. Hence it's utility. */
    private def onChange(oldCa: Int, ca: Int | String): Unit = {
      val p = $.props.runNow()

      val value = (ca: Any) match {
        case i: Int                 => i
        case s: String if s.isEmpty => p.min
        case s: String              => filterChars(s).toInt
        case _                      => p.max
      }

      val boundedValue = (value min p.max) max p.min

      // Makeshift tristate
      val isUp =
        if (boundedValue == oldCa) None
        else if (boundedValue < oldCa) Some(false)
        else Some(true)

      val result = if (boundedValue == p.max || boundedValue == p.min) {
        Some(boundedValue)
      } else {
        isUp match {
          case Some(yes) =>
            p.options.sliding(2).collectFirst {
              case Vector(h, b) if boundedValue > h && boundedValue < b =>
                if (yes) b
                else h
            }
          case None => None
        }
      }
      result foreach { f =>
        setFields(FieldOptions.tupledTouched(p.name, f))
      }
    }

    def value: UndefOr[Int] = {
      getFieldValueCoerce[Long]($.props.runNow().name).map(_.toInt)
    }

    def render(p: Props) = {
      // Better method?
      val fieldValue = value.getOrElse(p.default)

      Form.Item(new Item.Opt {})(Row(new Row.Opt {
        override val `type`    = "flex"
        override val justify   = "space-around"
        override val align     = "middle"
        override val className = "dni-row"
      }.ex(p.tagMods: _*))(Icon(new Icon.Opt {
        val `type`            = "minus-circle-o"
        val className         = "dni-change"
        override val disabled = fieldValue <= p.min
      }.ex(^.onClick ==> onIncDec(false))), getFieldDecorator(p.name)(<.h2(p.fmt(fieldValue))), Icon(new Icon.Opt {
        val `type`            = "plus-circle-o"
        val className         = "dni-change"
        override val disabled = fieldValue >= p.max
      }.ex(^.onClick ==> onIncDec(true)))))
    }
  }

  val C = ScalaComponent
    .builder[Props]("DiscreteNumericInput")
    .stateless
    .renderBackend[Backend]
    .componentWillMount(_.backend.mount)
    .build
}
