package web.cps

import ant.cps._
import ant.cps.Form.{FieldOptions, FormBackend, FormComponent}
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.Attr
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import org.scalajs.dom
import relay.components._
import relay.graphql.TaggedNode
import shared.models.Flds
import shared.proto.changeevents.NameChange
import web.base.data.{ProposedChangeFrag, UserFrag, ViewerFrag}
import web.base.all._
import web.mutation.{CancelChangeMutation, ChangeMutation}
import web.views.Settings

import scala.concurrent.Future
import scala.scalajs.js
import scala.scalajs.js.annotation.ScalaJSDefined
import scala.util.control.NonFatal
import scala.util.{Failure, Success}
import shared.proto.TaggedClassSerDe._
import web.base.func.Showable
import web.react.RelayBackend

object NameSettings {
  import web.views.Settings._

  @js.native
  private trait NamesVF extends js.Object {
    val firstName: String
    val lastName: String
  }

  case class State(flash: Option[(String, VdomElement)] = None, loading: Boolean = false) {
    def setFlash(status: String, message: VdomElement): State = copy(Some((status, message)))
    def toggleLoading: State                                  = copy(loading = !loading)
  }

  class Backend(val $ : BackendScope[Unit, State]) extends RelayBackend[HOCProps] with FormBackend {

    def submit(e: ReactEvent) =
      e.preventDefaultCB >> Callback {
        validateFields[NamesVF]()
          .flatMap {
            case Left(f) => Callback.warn(f).toTask
            case Right(r) =>
              $.modState(_.toggleLoading).toTask.flatMap { _ =>
                ChangeMutation
                  .commit(relay.environment, relayProps.viewer.id, NameChange(now, r.firstName, r.lastName)) flatMap {
                  case Right(f) if f.proposeChange.proposedChange.isEmpty =>
                    $.modState(
                      _.setFlash("success",
                                 <.span("No changes made, email us if this is in error at ",
                                        <.a(^.href := "mailto:support@joincollective.com")(
                                          "support@joincollective.com")))).toTask
                  case Right(_) => Task.unit
                  case Left(l)  => Task(form.setFields(l.jsErrors))
                }
              }

          }
          .onErrorHandle { case NonFatal(f) => f.printStackTrace() }
          .flatMap { _ =>
            $.modState(_.toggleLoading).toTask
          }
          .runAsyncForget
      }.void

    //======================= Event handlers
    def onPendingClose(id: String)(rei: ReactEvent) = rei.preventDefaultCB >> Callback {
      CancelChangeMutation.commit(relay.environment, relayProps.viewer.id, id :: Nil) runOnComplete {
        case Success(Left(f))  => dom.console.error(f.jsErrors)
        case Success(Right(_)) => ()
        case Failure(t)        => t.printStackTrace()
      }
    }

    def pendingChangesMessage(changes: js.Array[(ProposedChangeFrag, NameChange)]): VdomNode = {

      if (changes.length == 0) VdomNode.cast(null)
      else {
        val num = changes.length
        val msg = <.span(s"You have $num change${if (num > 1) "s" else ""} pending. ",
                         <.div(changes.toVdomArray {
                           case (n, after) =>
                             Row(`type` = "flex", key = n.id)(<.div(^.marginTop := "10px")(Card.C(new Card.Opts {})(
                               <.strong("Proposed Name Change - "),
                               <.code(n.rawId),
                               Row()(Col(new Col.Opt {
                                       override val offset = 1
                                     })(<.div("First Name: ", after.firstName), <.div("Last Name: ", after.lastName)),
                                     Button(Button.Opt.ex(^.onClick ==> onPendingClose(n.rawId),
                                                          ^.marginTop := "10px"))("Cancel")))))
                         }))
        Alert(showIcon = true, message = js.defined(msg.rawNode), closable = false)
      }
    }

    def render(s: State) = {
      val changes: js.Array[(ProposedChangeFrag, NameChange)] = relayProps.viewer.changes.edges
        .flatMap(n => n.node.after.toGMsg.map((n.node, _)))
        .collect { case (node, n: NameChange) => (node, n) }

      val disabled = changes.length > 0

      <.div(pendingChangesMessage(changes),
            s.flash.fold(VdomNode.cast(null)) {
              case (status, message) =>
                Alert(`type` = status, showIcon = true, message = message.rawNode, closable = true)
            },
            Form((new Form.Opt {
              override val layout           = "horizontal"
              override val hideRequiredMark = true
            })(^.onSubmit ==> submit))(
              Form.Item(style.respItem("First Name"))(getFieldDecorator(Flds.firstName, new FormComponent.Opt {
                override val initialValue = relayProps.viewer.user.map(_.user.firstName)
              })(Input(placeholder = "First Name", disabled = disabled))),
              Form.Item(style.respItem("Last Name"))(getFieldDecorator(Flds.lastName, new FormComponent.Opt {
                override val initialValue = relayProps.viewer.user.map(_.user.lastName)
              })(Input(placeholder = "Last Name", disabled = disabled))),
              //
              Form.Item(style.respTailItem)(
                Button(className = "jc-button", `type` = "dashed")("Reset"),
                Button(className = "jc-button", htmlType = "submit", `type` = "primary", loading = s.loading)("Save"))))
    }

  }

  trait HOCProps extends js.Object {
    val viewer: ViewerFrag
  }

  /**
    * This NameSettings_changes is used by mutations
    */
  @gql("""
    fragment NameSettings_viewer on Viewer {
      id
      user {
        id
        user {
          id
          firstName
          lastName
        }
      }
      changes(
        after: $after,
        first: $first
      ) @connection(key: "NameSettings_changes") {
          edges {
            node {
              id
              rawId
              when
              before
              after
            }
            cursor
          }
      }
    }
    """)
  object viewer_change extends TaggedNode

  private val C =
    ScalaComponent.builder[Unit]("NameSettings").initialState(State()).renderBackend[Backend].buildRelay[HOCProps]

  val FormC = RelayForm
    .builder[js.Object]
    .wrap(C)
    .mapPropsToFieldsS { p =>
      p.viewer.user.fold(Seq.empty[(String, FieldOptions)]) { fu =>
        Seq(FieldOptions.tupled(Flds.firstName, fu.user.firstName),
            FieldOptions.tupled(Flds.lastName, fu.user.lastName))
      }
    }
    .build

  val Container = FragmentContainer.buildRelay(FormC, "viewer" -> viewer_change)
}
