package web.cps

import ant.cps.Form.{FieldOptions, FormComponent, FormFacade, FormProps}
import ant.cps.{Icon, Row}
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import web.base.all._

import scala.scalajs.js.{|, UndefOr}

object SteppedNumInput {

  case class Props(formProps: FormProps,
                   name: String,
                   upFn: Int => Int,
                   downFn: Int => Int,
                   default: Int,
                   min: Int           = 0,
                   max: Int           = Int.MaxValue,
                   fmt: Int => String = _.toString)

  val C = ScalaComponent
    .builder[Props]("SteppedNumInput")
    .stateless
    .renderBackend[Backend]
    .componentWillMount(_.backend.mount)
    .build

  class Backend(val $ : BackendScope[Props, Unit]) extends FormFacade {

    def form: FormProps = $.props.runNow().formProps

    //============== Lifecycle events =========================
    def mount: Callback = $.props.map { p =>
      val fieldValue = value.getOrElse(p.default)
      form.getFieldDecorator(p.name, new FormComponent.Opt {
        override val initialValue = fieldValue
      })
      ()
    }

    private def onIncDec(up: Boolean): ReactEvent => Callback = { ri =>
      ri.preventDefaultCB >> $.props.map { p =>
        value.foreach { v =>
          val ud =
            if (up) p.upFn(v)
            else p.downFn(v)
          onChange(v, ud)
        }
      }
    }

    private def onChange(oldCa: Int, value: Int): Unit = {
      val p = $.props.runNow()

      val boundedValue = (value min p.max) max p.min

      // Makeshift tristate

      val result = if (boundedValue == p.max || boundedValue == p.min) {
        boundedValue
      } else value

      if (result != oldCa) {
        setFields(FieldOptions.tupledTouched(p.name, result))
      }
    }

    def value: UndefOr[Int] = {
      getFieldValueCoerce[Long]($.props.runNow().name).map(_.toInt)
    }

    def render(p: Props) = {
      // Better method?
      val fieldValue = value.getOrElse(p.default)

      Row(new Row.Opt {
        override val `type`    = "flex"
        override val justify   = "space-around"
        override val align     = "middle"
        override val className = "dni-row"
      })(Icon(new Icon.Opt {
        val `type`            = "minus-circle-o"
        val className         = "dni-change"
        override val disabled = fieldValue <= p.min
      }.ex(^.onClick ==> onIncDec(false))), <.h2(p.fmt(fieldValue)), Icon(new Icon.Opt {
        val `type`            = "plus-circle-o"
        val className         = "dni-change"
        override val disabled = fieldValue >= p.max
      }.ex(^.onClick ==> onIncDec(true))))
    }
  }
}
