package web.cps.a1quote

import java.text.DecimalFormatSymbols

import ant.cps._
import ant.cps.Form.{FieldOptions, FormBackend, FormComponent, Item}
import ant.cps.FormCreate.TypedFieldOptions
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.execution.misc.NonFatal
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import nprogress.NProgress
import org.scalajs.dom
import vdom.html_<^._
import relay.components._
import relay.runtime.store.RelayEnvironment
import rmi.ReactMaskedInput
import shared.models.a1.FieldKey._
import shared.models.a1.{DataCache, GuardTheftEnum, FieldKey => Fk}
import shared.proto.base.RawField
import web.base.all._
import web.base.data.{FieldSerializer, JsCoerce, RawFieldSerializer}
import web.cps.{style, DiscreteNumericInput, FormMaskedInput}
import web.mutation.QuoteA1BuilderM
import shared.proto.TaggedClassSerDe._
import shared.proto.a1.quote.NextStep
import web.routes.Routes.{Page, QuoteA1Page}
import web.views.Clean

import scala.scalajs.js.JSConverters._
import scala.concurrent.Future
import scala.scalajs.js
import scala.scalajs.js.|

object QuoteA1Form {
  import DiscreteNumericInput.{formatNum, formatUSD}

  //
  case class Props(ctl: RouterCtl[Page],
                   f: TypedFrag[QuoteA1Frag],
                   env: RelayEnvironment,
                   queue: PublishSubject[(Fk[_], TypedFieldOptions[js.Any])])

  /**
    * Parse a field and do something with it.  We specifically deal with the touched
    * attribute that says I am ready to submit.
    * @param fk
    * @param tfo
    * @return
    */
  def parseField(fk: Fk[_], tfo: TypedFieldOptions[js.Any]): Option[RawField] = {

    def f[X](fk: Fk[X])(implicit X: RawFieldSerializer[X], T: JsCoerce[X]): Option[RawField] = {
      T.convert(tfo.value) map { x =>
        FieldSerializer(fk, x)
      }
    }

    if (tfo.touched.exists(identity)) {
      fk match {
        case p @ CovC                  => f(p)
        case p @ CovD                  => f(p)
        case p @ CovE                  => f(p)
        case p @ CovF                  => f(p)
        case p @ Deduct                => f(p)
        case p @ Zip                   => f(p)
        case p @ NumStories            => f(p)
        case p @ SecuritiesSL          => f(p)
        case p @ JewelrySL             => f(p)
        case p @ MoneySL               => f(p)
        case p @ SilverwareSL          => f(p)
        case p @ FirearmsSL            => f(p)
        case p @ ElectronicApparatusSL => f(p)
        case p @ TapesRecordsSL        => f(p)
        case p @ PootersSL             => f(p)
        case p @ FireExt               => f(p)
        case p @ SmokeAlarm            => f(p)
        case p @ Sprinkler             => f(p)
        case p @ FireAlarm             => f(p)
        case p @ Guard                 => f(p)
        case p @ TheftAlarm            => f(p)
        case p @ Deadbolt              => f(p)
        case p @ SwimmingP             => f(p)
        case p @ Replacement           => f(p)
        case _ =>
          dom.console.log("Unhandled", fk.value, tfo)
          None
      }
    } else {
      None
    }
  }

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    val c = SingleAssignmentCancelable() // Fwd reference.

    def willMount(): Callback = {
      $.props.map { p =>
        c := p.queue.subscribe(new Observer[(Fk[_], TypedFieldOptions[js.Any])] {
          def onError(ex: Throwable) = ex.printStackTrace()
          def onComplete()           = println("complete!")
          def onNext(elem: (Fk[_], TypedFieldOptions[js.Any])) = {
            val (fk, tfo) = elem
            parseField(fk, tfo).fold[Future[Ack.Continue]](Ack.Continue) { rf =>
              NProgress.start()
              QuoteA1BuilderM.commit(p.env, rf, tfo).runAsync.flatMap { _ =>
                NProgress.done()
                // The result of each individual commit is captured in the `l` parameter
                Ack.Continue
              }
            }
          }
        })
        ()
      }
    }

    val onSub = (re: ReactEvent) =>
      re.preventDefaultCB >> $.props >>= { p =>
        Callback {
          NProgress.start()
          QuoteA1BuilderM.commit(p.env, NextStep(2)).map { _ =>
            NProgress.done()
            // The result of each individual commit is captured in the `l` parameter
            $.props
              .flatMap { p =>
                if (p.f.r.step.get != 1) {
                  dom.console.log(s" SUB ${p.f.r.step}")
                  p.ctl.set(QuoteA1Page(Some(2)))
                } else Callback.empty
              }
              .runNow()

            Ack.Continue
          }
        } >> Callback.log("Submitted")
    }

    def render(p: Props) = {
      Form(new Form.Opt {
        override val layout   = "horizontal"
        override val onSubmit = onSub.toJsCallback
      })(
        Clean.BareRow(
          Clean.AccentHeader(<.h1(^.color := "white")("Renter's insurance for California!")),
          Clean.BareBody(
            <.h2("High Impact Stuff"),
            <.p(^.marginBottom := "12px")(
              "These selections and factors tend to change your coverage and price the most. " +
                "Make sure to pay attention to what you put here!"),
            FormMaskedInput.C(FormMaskedInput.Props(form, Fk.Zip.value, new ReactMaskedInput.Opts {
              override val mask        = "11111"
              override val placeholder = "5 digit zip"
            }, style.respItem("Enter your Zipcode"))),
            Item(style.respItem("How much stuff do you have?"))(
              getFieldDecorator(Fk.CovC.value)(
                DiscreteNumericInput.C(
                  DiscreteNumericInput.Props(
                    form,
                    Fk.CovC.value,
                    DataCache.coverageC.head,
                    DataCache.coverageC.last,
                    DataCache.coverageC,
                    DataCache.coverageC(6),
                    fmt = formatUSD)))),
            Item(style.respItem("Deductible size?"))(
              getFieldDecorator(Fk.Deduct.value)(Radio.Group()(DataCache.deductible.toVdomArray(f =>
                Radio.Button(value = f, key = f.toString)(formatUSD(f)))))),
            Item(style.respItem("Liability coverage?"))(
              getFieldDecorator(Fk.CovE.value)(Radio.Group()(DataCache.coverageE.toVdomArray(f =>
                Radio.Button(value = f, key = f.toString)(formatUSD(f)))))),
            Item(style.respItem("How would you like your stuff replaced?"))(getFieldDecorator(Fk.Replacement.value)(
              Radio.Group()(Radio.Button(value = false)("Craigslist"), Radio.Button(value = true)("Amazon")))),
            <.h2("Lower Impact (but still important!)"),
            <.p(^.marginBottom := "12px")("These options still impact your coverage, and reflect in your price."),
            Item(style.respItem("Number of stories?"))(
              getFieldDecorator(Fk.NumStories.value)(Radio.Group()(DataCache.numStories.toVdomArray(f =>
                Radio.Button(value = f, key = f)(f))))),
            Item(style.respItem("Medical payments to others?"))(
              getFieldDecorator(Fk.CovF.value)(Radio.Group()(DataCache.coverageF.toVdomArray(f =>
                Radio.Button(value = f, key = f.toString)(formatUSD(f)))))),
            Item(style.respItem("Swimming pool?"))(getFieldDecorator(Fk.SwimmingP.value)(
              Radio.Group()(Radio.Button(value = false)("No =("), Radio.Button(value = true)("Yes, it's awesome")))),
            <.h2("Special Items"),
            <.p(^.marginBottom := "12px")("These classes of items have special limits detailed in the forms. " +
              "They help us price the risk better which makes the pool more efficient."),
            SpecialItemsControl.C(SpecialItemsControl.Props(form)),
            <.h2("Discounts, let us know if you qualify for discounts."),
            <.p("Fire alarms, sprinklers, security all help prevent losses, let us know which ones you have " +
              "and we can price your plan better."),
            Item(style.respItem("Fire Extinguisher?"))(
              //
              getFieldDecorator(Fk.FireExt.value)(
                Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
            Item(style.respItem("Smoke Alarm?"))(
              //
              getFieldDecorator(Fk.SmokeAlarm.value)(
                Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
            Item(style.respItem("Fire Alarm?"))(
              //
              getFieldDecorator(Fk.FireAlarm.value)(Radio.Group()(
                Radio.Button(value = "No")("None"),
                Radio.Button(value = "Local")("Local"),
                Radio.Button(value = "Direct")("Direct"),
                Radio.Button(value = "Central")("Central"),
              ))),
            Item(style.respItem("Sprinkler in your unit?"))(
              //
              getFieldDecorator(Fk.Sprinkler.value)(
                Radio.Group()(
                  Radio.Button(value = "No")("No"),
                  Radio.Button(value = "Partial")("Partial"),
                  Radio.Button(value = "Full")("Full")))),
            Item(style.respItem("Deadbolt?"))(
              //
              getFieldDecorator(Fk.Deadbolt.value)(
                Radio.Group()(Radio.Button(value = false)("No"), Radio.Button(value = true)("Yes")))),
            Item(style.respItem("Do you have a Theft Alarm?"))(
              //
              getFieldDecorator(Fk.TheftAlarm.value)(Radio.Group()(
                Radio.Button(value = "No")("None"),
                Radio.Button(value = "Local")("Local"),
                Radio.Button(value = "Direct")("Direct"),
                Radio.Button(value = "Central")("Central"),
              ))),
            Item(style.respItem("Do you have a Guard?"))(
              //
              getFieldDecorator(Fk.Guard.value)(Radio.Group()(GuardTheftEnum.values.toVdomArray(f =>
                Radio.Button(value = f.value, key = f.value)(f.value))))),
            Item(style.respTailItem)(
              Button(`type` = "ghost", className   = "jc-button")("Cancel"),
              Button(`type` = "primary", className = "jc-button", htmlType = "submit")("Next Step")))))
    }
  }

  val C = ScalaComponent
    .builder[Props]("QA1Form")
    .stateless
    .renderBackend[Backend]
    .componentWillMount(_.backend.willMount())
    .build

  val FormC = FormCreate
    .builder[js.Dictionary[TypedFieldOptions[js.Any]]](C)
    .onFieldsChange { (p, f) =>
      f.toVector.flatMap(key => Fk.withValueOpt(key._1) map (_ -> key._2)) foreach { v =>
        p.unbox.queue.onNext(v)
      }
    }
    .mapPropsToFields { p =>
      dom.console.log(p.unbox.f.r)
      val r: Seq[(String, TypedFieldOptions[js.Any])] = Seq(
        p.unbox.f.pair(Fk.CovC),
        p.unbox.f.pair(Fk.Deduct),
        p.unbox.f.pairOpt(Fk.CovE),
        p.unbox.f.pairOpt(Fk.CovF),
        p.unbox.f.pair(Fk.Zip),
        p.unbox.f.pair(Fk.NumStories),
        p.unbox.f.pair(Fk.SecuritiesSL),
        p.unbox.f.pair(Fk.JewelrySL),
        p.unbox.f.pair(Fk.MoneySL),
        p.unbox.f.pair(Fk.SilverwareSL),
        p.unbox.f.pair(Fk.FirearmsSL),
        p.unbox.f.pair(Fk.ElectronicApparatusSL),
        p.unbox.f.pair(Fk.TapesRecordsSL),
        p.unbox.f.pair(Fk.PootersSL),
        p.unbox.f.pairConvert(Fk.FireAlarm)(_.value),
        p.unbox.f.pairConvert(Fk.Sprinkler)(_.value),
        p.unbox.f.pair(Fk.FireExt),
        p.unbox.f.pair(Fk.SmokeAlarm),
        p.unbox.f.pairConvert(Fk.TheftAlarm)(_.value),
        p.unbox.f.pairConvert(Fk.Guard)(_.value),
        p.unbox.f.pair(Fk.Deadbolt),
        p.unbox.f.pair(Fk.SwimmingP),
        p.unbox.f.pair(Fk.Replacement),
      )

      val dict = js.Dictionary(r: _*)
      dom.console.log(dict)
      dict
    }
    .build

//  val Container = FragmentContainer.buildRelay(C, "qa1" -> QuoteA1View.quotef)

}
