package web.cps.a1quote

import ant.cps.FormCreate.TypedFieldOptions
import ant.cps.Layout.{Content, Sider}
import ant.cps.Menu.SelectMenuCallback
import ant.cps._
import dispalt.relay.gql
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react._
import japgolly.scalajs.react.component.Scala
import monix.reactive.subjects.PublishSubject
import org.scalajs.dom
import org.scalajs.dom.{MediaQueryList, MediaQueryListListener}
import relay.QueryRenderer
import relay.components._
import relay.generated.{QuotePayload, QuotePayload_qa1}
import relay.graphql.TaggedNode
import relay.runtime.store.RelayEnvironment
import shared.models.a1.FieldKey
import shared.proto.base.RawField
import vdom.html_<^._
import web.cps.{PageFooter, PageSpinner}
import web.mutation.QuoteA1BuilderM
import web.react.RelayBackend
import web.routes.Routes.{Page, QuoteA1Page}
import web.base.all._

import scala.scalajs.js
import scala.scalajs.js.UndefOr
import shared.proto.TaggedClassSerDe._
import web.cps.style.Stat
import web.routes.Routes
import web.views.Clean

object QuoteA1View {

  case class Props(ctl: RouterCtl[Page], page: QuoteA1Page)
  case class State(step: Int, below: Boolean, collapsed: Boolean) {
    def toggleCollapsed: State = copy(collapsed = !collapsed)
  }

  // =============== Component Helpers
  def newRow(ch: VdomNode): VdomNode = { <.div(^.padding := "12px")(ch) }

  class Backend(val $ : BackendScope[Props, State]) extends RelayBackend[QuoteA1Frag] {

    @inline def qa1 = relayProps.qa1

    //=============== Init ================
    val queue = PublishSubject[(FieldKey[_], TypedFieldOptions[js.Any])]()
    val mq    = dom.window.matchMedia("(max-width: 992px)")
    def mount =
      Callback {
        mq.addListener(mqListener)
        mqListener(mq)
      } >> $.props.flatMap { p =>
        val step = qa1.r.step.get
        if (p.page.step.isEmpty)
          p.ctl.set(QuoteA1Page(Some(step)))
        else if (p.page.step.exists(_ > step))
          p.ctl.set(QuoteA1Page(Some(step)))
        else Callback.empty
      }

    def receiveProps(current: Props, next: Props, raw: Scala.RawMounted[Props, State, Backend]): Callback = Callback {
      //
      dom.console.log(qa1.r.step)
    }

    def unmount = Callback(mq.removeListener(mqListener))

    //======================= Event Handlers =============
    val mqListener: MediaQueryListListener = { mql: MediaQueryList =>
      $.modState(_.copy(below = mql.matches)) >>
        $.state.flatMap { s =>
          if (s.collapsed != mql.matches) {
            $.modState(_.toggleCollapsed)
          } else Callback.empty
        }
    }.toJsCallback.asInstanceOf[MediaQueryListListener]

    val selectMenu = (p: SelectMenuCallback) =>
      p.key match {
        case "home"  => $.props.flatMap(_.ctl.set(Routes.Root))
        case "reset" => Callback.empty
    }

    // ===================== Modular ==================
    /* Helpers on vdom */
    def header(collapsed: Boolean): VdomElement = {
      val icn = if (collapsed) "menu-unfold" else "menu-fold"
      <.div(
        Icon(new Icon.Opt {
          val `type` = icn
        }.ex(^.onClick --> $.modState(_.toggleCollapsed), ^.id := "trigger")),
        Menu(new Menu.Opt {
          override val mode     = "horizontal"
          override val id       = "nav"
          override val onSelect = selectMenu.toJsCallback

        }.ex(^.backgroundColor := "transparent"))(Menu.Item(key = "home")(<.span(Icon("home")), "Home"),
                                                  Menu.Item(key = "reset")(PopConfirm(new PopConfirm.Opt {
                                                    override val title     = "Are you sure you want to start over?"
                                                    override val placement = "bottomRight"
                                                    override val onConfirm =
                                                      js.defined(Callback.log("onConfirm").toJsFn)
                                                  })(<.div(Icon("reload"), "Redo")))))
    }

    //======================== Render =================

    def render(p: Props, s: State) = {
      val step = p.page.step.getOrElse(1)

      //
      Layout(new Layout.Opts {}
        .ex(^.minHeight := "100vh"))(
        Sider(new Sider.Opts {
          override val collapsedWidth = 0
          override val collapsible    = true
          override val trigger        = null
          override val width          = 300
          override val collapsed      = s.collapsed
        }.ex(^.background := "#fff", ^.borderRight := "1px solid #ececec"))(<.div(^.overflow := "hide")(
          newRow(Progress(new Progress.Opt {
            override val status  = "active"
            override val percent = qa1.r.progress
            //
          })),
          <.div(^.padding := "0 12px")(qa1.r.price map { p =>
            Row(`type` = "flex", justify = "space-around", align = "center")(Stat(p, None, Some("$/year"), None)).vdomElement
          } getOrElse <.div("")))),
        Layout(new Layout.Opts {}.ex(^.background := "linear-gradient(120deg, #89f7fe 0%, #66a6ff 100%)"))(
          Content(Clean.Bare(Clean.Props(Some(header(s.collapsed)), footer = false)) {
            if (step == 1)
              QuoteA1Form.FormC(QuoteA1Form.Props(p.ctl, qa1, relay.environment, queue))
            else
              QuoteA1Page2.FormC(QuoteA1Page2.Props(qa1, relay.environment, queue))
          }),
          PageFooter.C(())))
    }
  }

  // test1
  @gql("""
    fragment QuotePayload_qa1 on QuotePayload {
      id
      cov_c cov_d cov_e cov_f
      deductible
      build_add
      zipcode
      num_stories

      # Special Items
      securities_sl jewelry_sl money_sl silverware_sl firearms_sl electronic_apparatus_sl tapes_records_sl
      pooters_sl

      fire_alarm sprinkler fire_ext smoke_alarm

      guard theft_alarm deadbolt

      swimming_pool
      replacement_cost

      household contents_quality

      l { key msg }
      price
      progress
      step
    }
  """)
  object quotef extends TaggedNode

  @gql("""
    query QuotePayload {
      qa1 {
        ...QuotePayload_qa1
      }
    }
  """)
  private object query extends TaggedNode

  val C = ScalaComponent
    .builder[Props]("QA1")
    .initialStateCallbackFromProps[State](p => CallbackTo(State(p.page.step.getOrElse(1), false, false)))
    .renderBackend[Backend]
    .componentWillMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .componentWillReceiveProps(f => f.backend.receiveProps(f.currentProps, f.nextProps, f.raw))
    .buildRelay[QuoteA1Frag]

  val Container = FragmentContainer.buildRelay(C, "qa1" -> QuotePayload_qa1)

  // ===================== Helpers for data fetching. =============
  private val spinner = PageSpinner.C()

  /**
    * Seperate query renderer since we don't want to do anything about login or bad
    * auth, since this is an authless workflow.
    *
    * @param ctl
    * @param env
    * @param page
    * @return
    */
  def QR(ctl: RouterCtl[Page], env: RelayEnvironment, page: QuoteA1Page) =
    QueryRenderer
      .builder[QuoteA1Frag](environment = env, query = QuotePayload)
      .renderEither {
        case Some(Left(queryError)) =>
          val err = queryError.errors
          if (err.isDefined) {
            dom.console.error("Server issue", err.get)
            Alert(message = s"Sorry we encountered a problem")
          } else {
            dom.console.error("Unknown issue", queryError)
            <.div("Unknown issue.")
          }

        case Some(Right(props)) =>
          Container.rprops(props)(Props(ctl, page))

        case None =>
          spinner
      }
      .build
}
