package web.cps.a1quote

import ant.cps.FormCreate.TypedFieldOptions
import ant.cps.{Alert, Icon, Menu, PopConfirm, Progress, Row, Steps}
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import monix.reactive.subjects.PublishSubject
import moment.Date
import org.scalajs.dom
import org.scalajs.dom.{MediaQueryList, MediaQueryListListener}
import relay.QueryRenderer
import relay.components._
import relay.generated.{QuoteParent_qa1, QuoteParent_viewer}
import relay.graphql.TaggedNode
import relay.runtime.store.RelayEnvironment
import shared.models.a1.{FieldKey => Fk}
import shared.proto.base.RawField
import shared.proto.a1.quote._
import shared.proto.TaggedClassSerDe._
import web.cps.{bareLo, AdjustingHeader, DiscreteNumInput, PageFooter, PageSpinner}
import web.react.RelayBackend
import web.routes.Routes.{Page, QuotePage}
import web.base.all._
import web.base.data.{FieldSerializer, JsCoerce, RawFieldSerializer}
import web.mutation.QuoteA1BuilderM

import scala.scalajs.js
import scala.scalajs.js.annotation.JSImport

object QuoteParent {

  final val headerHeight = 48

  case class CProps(f: TypedFrag[QuoteA1Frag],
                    env: RelayEnvironment,
                    queue: PublishSubject[(Fk[_], TypedFieldOptions[js.Any])],
                    lgToMd: Boolean,
                    mdTo0: Boolean)

  case class Props(ctl: RouterCtl[Page], page: QuotePage) {
    def step = page.step.getOrElse(1)
  }
  case class State(lgToMd: Boolean, mdTo0: Boolean)

  /**
    * Parse a field and do something with it.  We specifically deal with the touched
    * attribute that says I am ready to submit.
    * @param fk
    * @param tfo
    * @return
    */
  def parseField(fk: Fk[_], tfo: TypedFieldOptions[js.Any]): Option[RawField] = {
    import Fk._

    def f[X](fk: Fk[X])(implicit X: RawFieldSerializer[X], T: JsCoerce[X]): Option[RawField] = {
      dom.console.log(s"Sending $fk", tfo)
      T.convert(tfo.value) map { x =>
        FieldSerializer(fk, x)(X)
      }
    }

    if (tfo.validating.exists(identity)) {
      None
    } else if (tfo.touched.exists(identity)) {
      fk match {
        case p @ CovC                  => f(p)
        case p @ CovD                  => f(p)
        case p @ CovE                  => f(p)
        case p @ CovF                  => f(p)
        case p @ Deduct                => f(p)
        case p @ Zip                   => f(p)
        case p @ NumStories            => f(p)
        case p @ SecuritiesSL          => f(p)
        case p @ JewelrySL             => f(p)
        case p @ MoneySL               => f(p)
        case p @ SilverwareSL          => f(p)
        case p @ FirearmsSL            => f(p)
        case p @ ElectronicApparatusSL => f(p)
        case p @ TapesRecordsSL        => f(p)
        case p @ PootersSL             => f(p)
        case p @ FireExt               => f(p)
        case p @ SmokeAlarm            => f(p)
        case p @ Sprinkler             => f(p)
        case p @ FireAlarm             => f(p)
        case p @ Guard                 => f(p)
        case p @ TheftAlarm            => f(p)
        case p @ Deadbolt              => f(p)
        case p @ SwimmingP             => f(p)
        case p @ Replacement           => f(p)
        case p @ Household             => f(p)
        case p @ ContentsQuality       => f(p)
        case p @ PlaceLoc              => f(p)
        case p @ PolicyStart           => f(p)
        case p @ Birthday              => f(p)

        // Handle these different
        case FirstName | LastName         => None
        case Email | Password | Password2 => None
        case _ =>
          dom.console.log("Unhandled", fk.value, tfo)
          None
      }
    } else {
      None
    }
  }

  class Backend(val $ : BackendScope[Props, State]) extends RelayBackend[QuoteA1Frag] {

    @inline def qa1: TypedFrag[QuoteA1Frag] = relayProps.qa1
    val queue                               = PublishSubject[(Fk[_], TypedFieldOptions[js.Any])]()

    val lgToMd: MediaQueryList = dom.window.matchMedia("(max-width: 1199px) and (min-width: 992px)")
    val mdTo0: MediaQueryList  = dom.window.matchMedia("(max-width: 992px)")

    val lgToMdListener: MediaQueryListListener = mqListener((m, s) => s.copy(lgToMd = m))
    val mdTo0Listener: MediaQueryListListener  = mqListener((m, s) => s.copy(mdTo0  = m))

    def unmount: Callback =
      Callback(lgToMd.removeListener(lgToMdListener)) >> Callback(mdTo0.removeListener(mdTo0Listener))

    def mount: Callback =
      Callback {
        lgToMd.addListener(lgToMdListener)
        mdTo0.addListener(mdTo0Listener)
        lgToMdListener(lgToMd)
        mdTo0Listener(mdTo0)
      } >> $.props.flatMap { p =>
        val step = qa1.r.step.get
        if (p.page.step.isEmpty)
          p.ctl.set(QuotePage(Some(step)))
        else if (p.page.step.exists(_ > step))
          p.ctl.set(QuotePage(Some(step)))
        else Callback.empty
      }

    //======================= Event Handlers =============
    def mqListener(func: (Boolean, State) => State): MediaQueryListListener = { mql: MediaQueryList =>
      $.state.flatMap { s =>
        val newS = func(mql.matches, s)
        if (newS != s) $.setState(newS)
        else Callback.empty
      }
    }.toJsCallback.asInstanceOf[MediaQueryListListener]

    val resetConfirm: Callback = Callback {
      QuoteA1BuilderM
        .commit(relay.environment, Reset(ec.currentTimeMillis()))
        .map { _ =>
          $.props.flatMap(p => p.ctl.set(QuotePage(qa1.r.step.toOption)))
        }
        .runAsyncForget
    }

    //===================== Render oriented functions ===============
    def header(p: Props)(n: VdomNode*) = {
      val price: String =
        qa1.r.priceMonthly.map(d => d + "/mo").getOrElse("No Price Yet!")
      AdjustingHeader.C(
        AdjustingHeader.Props(
          headerHeight,
          <.div(^.textAlign.center, ^.fontSize := "1.75em")(<.span(price), Progress(new Progress.Opt {
            override val strokeWidth = 5

            override val `type`  = "circle"
            override val width   = headerHeight - 10
            override val percent = qa1.r.progress
          }.ex(^.marginLeft := "10px", ^.width := "120px"))),
          Menu(mode = "horizontal", id = "nav")(Menu.Item(key = "reset")(PopConfirm(new PopConfirm.Opt {
            override val title     = js.defined("Confirm you'd like to reset.")
            override val placement = "bottomRight"
            override val onConfirm = js.defined(resetConfirm.toJsFn)
          })(<.div(Icon("reload"), "Reset")))),
          adjust = false))(n: _*)
    }

    def renderConfirmation(p: Props, s: State): VdomElement = {
      Step5.C(Step5.Props(CProps(qa1, relay.environment, queue, lgToMd = s.lgToMd, mdTo0 = s.mdTo0), p))
    }

    def renderPage(p: Props, s: State): VdomElement = {
      p.page.step match {
        case _ | Some(5) if qa1.t.bound => renderConfirmation(p, s)
        case Some(5)                    => renderConfirmation(p, s)
        case Some(4) =>
          Step4.FormC(Step4
            .Props(CProps(qa1, relay.environment, queue, lgToMd = s.lgToMd, mdTo0 = s.mdTo0), p, relayProps.optViewer))
        case Some(3) =>
          Step3.FormC(Step3.Props(p, qa1, relay.environment, queue, lgToMd = s.lgToMd, mdTo0 = s.mdTo0))
        case Some(2) =>
          Step2.FormC(Step2.Props(p.ctl, p.page, qa1, relay.environment, queue, lgToMd = s.lgToMd, mdTo0 = s.mdTo0))
        case Some(1) | None =>
          Step1.FormC(Step1.Props(p.ctl, p.page, qa1, relay.environment, queue, lgToMd = s.lgToMd, mdTo0 = s.mdTo0))
        case _ => <.div("Hello")
      }
    }

    def render(p: Props, s: State) = {
      <.div(^.className := bareLo("bg"))(
        header(p)(
          Row(`type` = "flex", justify = if (s.lgToMd || s.mdTo0) "" else "center")(
            <.div(^.className := bareLo + " " + bareLo("container"))(<.div(^.className := bareLo("main-wrapper"))(
              Row(new Row.Opt {
                override val `type`  = "flex"
                override val justify = "center"
                override val align   = "middle"
              }.ex(^.margin := "30px 0 48px", ^.textAlign.center))(
                <.img(^.src := "/assets/images/02_general_expenses_64.png"),
                "Better Insurance through Science, not gimmicks."),
              Steps(
                p.page.step.map(_ - 1).getOrElse(0),
                size             = "small",
                direction        = if (s.mdTo0) "vertical" else "horizontal")(
                Steps.Step(title = "Basic Details"),
                Steps.Step(title = "Property"),
                Steps.Step(title = "Special"),
                Steps.Step(title = "Personal"),
                Steps.Step(title = "Confirm & 🎉"),
              ),
              renderPage(p, s)))),
          PageFooter()))
    }
  }
  // test

  @gql("""
    fragment QuoteParent_qa1 on QuotePayload {
      id
      # Coverage!
      cov_c cov_d cov_e cov_f cov_d_limit cov_d_default
      deductible
      build_add
      zipcode
      num_stories

      # Special Items
      securities_sl jewelry_sl money_sl silverware_sl firearms_sl electronic_apparatus_sl tapes_records_sl
      pooters_sl

      fire_alarm sprinkler fire_ext smoke_alarm

      guard theft_alarm deadbolt

      swimming_pool
      replacement_cost

      # Pseudo elements
      household contents_quality

      # Recommendations
      step1_rec {
        cov_d cov_c cov_e deductible cov_f
      }

      place_loc { lat lng formattedAddress }
      policy_start

      # Login stuff
      birthday first_name last_name email

      l { key msg }
      price priceMonthly
      progress
      step
      bound
    }
  """)
  object quotefrag extends TaggedNode

  @gql("""
       fragment QuoteParent_viewer on Viewer {
         user {
           id user { email firstName }
         }
       }
    """)
  object optViewer

  @gql("""
    query QuoteParent {
      qa1 {
        ...QuoteParent_qa1
      }
      optViewer {
        ...QuoteParent_viewer
      }
    }
  """)
  private object query extends TaggedNode

  //
  val C = ScalaComponent
    .builder[Props]("QuoteParent")
    .initialState(State(false, false))
    .renderBackend[Backend]
    .componentWillMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .buildRelay[QuoteA1Frag]

  val Container = FragmentContainer.buildRelay(C, "qa1" -> QuoteParent_qa1, "optViewer" -> QuoteParent_viewer)

  // ===================== Helpers for data fetching. ============
  private val spinner = PageSpinner.C()

  /**
    * Seperate query renderer since we don't want to do anything about login or bad
    * auth, since this is an authless workflow.
    *
    * @param ctl
    * @param env
    * @param page
    * @return
    */
  def QR(ctl: RouterCtl[Page], env: RelayEnvironment, page: QuotePage) =
    QueryRenderer
      .builder[QuoteA1Frag](environment = env, query = relay.generated.QuoteParent)
      .renderEither {
        case Some(Left(queryError)) =>
          val err = queryError.errors
          if (err.isDefined) {
            dom.console.error("Server issue", err.get)
            Alert(message = s"Sorry we encountered a problem")
          } else {
            dom.console.error("Unknown issue", queryError)
            <.div("Unknown issue.")
          }

        case Some(Right(props)) =>
          Container.rprops(props)(Props(ctl, page))

        case None =>
          spinner
      }
      .build
}
