package web.cps.a1quote

import ant.cps.Form.FormComponent.RuleOpt
import ant.cps.Form.{apply => _, _}
import ant.cps.FormCreate.TypedFieldOptions
import ant.cps._
import google.maps.LatLng
import google.maps.places.PlaceResult
import japgolly.scalajs.react._
import japgolly.scalajs.react.component.Scala.Unmounted
import japgolly.scalajs.react.vdom.html_<^._
import moment.Moment
import monix.eval.Task
import monix.execution.Ack
import monix.execution.cancelables.SingleAssignmentCancelable
import monix.reactive.Observer
import monix.reactive.subjects.PublishSubject
import nprogress.NProgress
import org.scalajs.dom
import rmi.ReactMaskedInput
import shared.models.ErrorKeys
import shared.models.a1.{DataCache, FieldKey => Fk}
import web.cps.a1quote.QuoteParent.CProps
import web.cps.a1quote.Step1.style
import web.cps.{bareLo, AddressInputMap, DiscreteNumInput, FormMaskedInput}
import web.models.PlaceOps
import web.base.all._
import web.mutation.QuoteA1BuilderM
import web.routes.Routes.QuotePage
import shared.proto.TaggedClassSerDe._
import shared.proto.a1.quote.{AccountRegDetails, CompleteRegistration}
import web.base.data.ViewerFrag
import web.cps.AddressInputMap.MarkerHelper

import scala.concurrent.Future
import scala.scalajs.js
import scala.util.Random

object Step4 {
  /* Initial center 1325 Guerrero St. */
  lazy val initialCenter = new LatLng(37.750015, -122.422530)

  case class Props(cp: CProps, qp: QuoteParent.Props, vf: OrNull[ViewerFrag])
  val C = ScalaComponent
    .builder[Props]("Step4")
    .stateless
    .renderBackend[Backend]
    .componentDidMount(_.backend.mount)
    .componentWillUnmount(_.backend.unmount)
    .build

  val FormC = FormCreate
    .defaultBuilder(C)
    .mapPropsToFields { p =>
      val f = p.unbox.cp.f

      js.Dictionary(
        Seq(
          f.pairConvert(Fk.PlaceLoc)(_.formattedAddress).copy(_1 = Fk.Place.value),
          f.pair(Fk.FirstName),
          f.pair(Fk.LastName),
          f.pair(Fk.Email),
          f.pair(Fk.Birthday),
          f.pairConvert(Fk.PolicyStart)(f => Moment(f)),
          f.err(ErrorKeys.serverFailure),
        ): _*)
    }
    .onFieldsChange { (p, f) =>
      f.toVector.flatMap(key => Fk.withValueOpt(key._1) map (_ -> key._2)) foreach { v =>
        p.unbox.cp.queue.onNext(v)
      }
    }
    .build

  def priceRow(cp: CProps): Unmounted[Unit, Unit, Unit] = {
    FormCtl.AccentRow(
      <.span("Your policy will be "),
      <.strong(cp.f.r.priceMonthly.get + "/month"),
      <.span(" (or "),
      <.strong(cp.f.r.price.get + "/year"),
      <.span(" paid annually)"))

  }

  class Backend(val $ : BackendScope[Props, Unit]) extends FormBackend {

    var validateAccount: Boolean = false
    //
    // ======================= Event handlers =========================
    val c: SingleAssignmentCancelable = SingleAssignmentCancelable()

    // =================== Helpers ===============================
    def mount: Callback = $.props.flatMap { p =>
      //
      Callback(c := p.cp.queue.subscribe(new Observer[(Fk[_], TypedFieldOptions[js.Any])] {
        def onNext(elem: (Fk[_], TypedFieldOptions[js.Any])): Future[Ack] = {
          val (fk, tfo) = elem
          QuoteParent.parseField(fk, tfo).fold[Future[Ack.Continue]](Ack.Continue) { rf =>
            NProgress.start()
            QuoteA1BuilderM.commit(p.cp.env, rf, tfo).runAsync.flatMap { _ =>
              NProgress.done()
              // The result of each individual commit is captured in the `l` parameter
              Ack.Continue
            }
          }
        }
        def onError(ex: Throwable): Unit = ex.printStackTrace()
        def onComplete(): Unit           = println("Complete")
      }))
    }

    def unmount: Callback = Callback(c.cancel())

    /* Thought here is to use a shadow field to handle it */
    def onPlaceChange(pr: PlaceResult): Callback = $.props.map { p =>
      PlaceOps.toPlaceCreated(Some(pr)) foreach { pc =>
        setFields(FieldOptions.tupledTouched(Fk.PlaceLoc, pc.asInstanceOf[js.Any]))
      }
    }

    def fieldNames: js.UndefOr[js.Array[String]] = {
      if (validateAccount) js.undefined
      else {
        js.Array[String](Fk.Place, Fk.PolicyStart, Fk.FirstName, Fk.LastName, Fk.Birthday)
      }
    }

    def goBack = $.props.flatMap(p => p.qp.ctl.set(QuotePage(Some(3))))

    val submit = (r: ReactEvent) =>
      r.preventDefaultCB >>
        $.props.map { p =>
          (for {
            _ <- Task(NProgress.start())
            cb <- formSubmitTrigger()
            r <- validateFieldsAndScrollT[js.Object](fieldNames)
            _ <- r match {
              case Left(err) => Task.unit
              case Right(r) =>
                val stf = new SimpleTypedFrag[js.Object](r)
                val cr = for {
                  fn <- stf(Fk.FirstName)
                  ln <- stf(Fk.LastName)
                } yield CompleteRegistration(fn, ln)

                val details =
                  if (!validateAccount) None
                  else
                    (for {
                      email <- stf(Fk.Email)
                      pw <- stf(Fk.Password)
                      pw2 <- stf(Fk.Password2)
                    } yield AccountRegDetails(email, pw, pw2)).toOption

                cr.fold(Task.unit) { cr =>
                  QuoteA1BuilderM.commit(p.cp.env, cr.copy(details = details)) flatMap { _ =>
                    $.props.toTask flatMap { pb =>
                      if (pb.cp.f.r.step.get >= 5) {
                        pb.qp.ctl.set(QuotePage(Some(5))).toTask
                      } else Task.unit
                    }
                  }
                }
            }
            _ <- Task(NProgress.done())
          } yield ()).runAsyncForget
      }

    def initial(p: Props) = {
      p.cp.f.r.place_loc.map { p =>
        val latLng = new LatLng(p.lat, p.lng)
        (latLng, MarkerHelper(p.formattedAddress, latLng))
      }.toOption
    }

    val whichShown: (Boolean) => Callback = (login: Boolean) => Callback(validateAccount = login)

    lazy val theName = DataCache.names(Random.nextInt(DataCache.names.length))

    def render(p: Props) = {
      Form(onSubmit = submit.toJsCallback)(
        <.div(^.className := bareLo("header"))(
          priceRow(p.cp),
          <.h1("Personal Policy Details"),
          <.p("""We need to get a couple last policy related details, and we are almost there!
                 Only at this point are we attempting to ask these details because we don't like giving out our
                 email to anyone and we suspect you don't either.
              """.stripMargin)),
        getFieldError[js.Array[String]](ErrorKeys.serverFailure)
          .fold(VdomNode.cast(null))(f => Alert(`type` = "error", message = f.mkString(", "))),
        //
        AddressInputMap.C {
          val init = initial(p)
          AddressInputMap.Props(
            form,
            init.map(_._2).toVector,
            init.map(_._1).getOrElse(initialCenter),
            p.cp.f.r.place_loc.toOption,
            onPlaceChange,
            None)
        },
        Form.Item(style.fullItem("Policy Start Date"))(getFieldDecorator(Fk.PolicyStart, new FormComponent.Opt {
          override val rules = js.defined(js.Array(new RuleOpt {
            val required: Boolean = true
            override val `type`   = "object"
            override val message  = "Please select a policy start date."
          }))
        })(DatePicker(new DatePicker.Opt {
          override val showToday = true
          override val format    = "MM/DD/YYYY"
        }))),
        <.div(^.className := bareLo("header"))(
          <.h1("Personal Details"),
          <.p("""
                |The price is set and unless you change something like your address, it will remain the same.
                |Also pay attention to the policy start date, it defaults to today, but note you can set it
                |in the future, if you plan on moving and we won't charge you until then.  We may however, authorize
                |the card.
              """.stripMargin)),
        FormCtl.splitRow(^.marginTop := "0px")(
          FormMaskedInput.C(FormMaskedInput.Props(form, Fk.Birthday, new ReactMaskedInput.Opts {
            override val mask        = "11/11/1111"
            override val placeholder = "MM/DD/YYYY"
          }, style.fullItem("Birthday")))),
        FormCtl.splitRow(^.marginTop := "0px")(
          Form.Item(style.fullItem("First Name"))(getFieldDecorator(Fk.FirstName, new FormComponent.Opt {
            override val validateTrigger = "onBlur"
            override val rules = js.defined(js.Array(new RuleOpt {
              val required: Boolean = true
              override val message  = "Please fill in your first name."
            }))
          })(Input(placeholder = theName._1))),
          Form.Item(style.fullItem("Last Name"))(getFieldDecorator(Fk.LastName, new FormComponent.Opt {
            override val validateTrigger = "onBlur"
            override val rules = js.defined(js.Array(new RuleOpt {
              val required: Boolean = true
              override val message  = "Please fill in your last name."
            }))
          })(Input(placeholder = theName._2)))),
        QuoteLoggedIn.C(QuoteLoggedIn.Props(p.vf, form, whichShown)),
        priceRow(p.cp),
        FormCtl.row(^.textAlign.center)(
          Row(`type` = "flex", justify = "space-around")(
            Button(onClick = goBack.toJsFn1)(Icon("left"), "Go Back"),
            Button(`type`  = "primary", htmlType = "submit")("Issue Policy", Icon("check")))))
    }
  }
}
