package web.cps.a1quote

import ant.cps.FormCreate.TypedFieldOptions
import org.scalajs.dom
import shared.models.a1.FieldKey
import web.base.data.{JsCoerce, LocalRateChecked}

import scala.scalajs.js
import scala.scalajs.js.{|, UndefOr}
import scala.scalajs.js.JSConverters._

class TypedFrag[T <: LocalRateChecked](val t: T) extends AnyVal {

  @inline def dyn: js.Dynamic = t.asInstanceOf[js.Dynamic]

  def apply[T1](fk: FieldKey[T1])(implicit toJsTyp: JsCoerce[T1]): js.UndefOr[T1] = {
    dyn.selectDynamic(fk.value).asInstanceOf[UndefOr[js.Any]].flatMap { f =>
      toJsTyp.convert(f).orUndefined
    }
  }

  def opt[T1](fk: FieldKey[Option[T1]])(implicit toJsTyp: JsCoerce[T1]): js.UndefOr[T1] = {
    dyn.selectDynamic(fk.value).asInstanceOf[UndefOr[js.Any]].flatMap { f =>
      toJsTyp.convert(f).orUndefined
    }
  }

  def pair[T1](fk: FieldKey[T1])(implicit toJsTyp: JsCoerce[T1]): (String, TypedFieldOptions[js.Any]) = {
    pairConvert(fk)((_: T1).asInstanceOf[js.Any])
  }

  def pairConvert[T1](fk: FieldKey[T1])(convert: T1 => js.Any)(
    implicit toJsTyp: JsCoerce[T1]): (String, TypedFieldOptions[js.Any]) = {
    val res = {
      val list = t.l.filter(_.key == fk.value).map(_.msg: js.Error | String)
      if (list.length == 0) js.undefined
      else js.defined(list)
    }

    val v = apply[T1](fk)(toJsTyp).map(convert).asInstanceOf[js.Any]

    val tfo: TypedFieldOptions[js.Any] = new TypedFieldOptions[js.Any] {
      val value               = v
      override val errors     = res
      override val touched    = false
      override val dirty      = true
      override val validating = false
    }
    fk.value -> tfo
  }

  def pairOpt[T1](fk: FieldKey[Option[T1]])(implicit toJsTyp: JsCoerce[T1]): (String, TypedFieldOptions[js.Any]) = {
    val res = {
      val list = t.l.filter(_.key == fk.value).map(_.msg: js.Error | String)
      if (list.length == 0) js.undefined
      else js.defined(list)
    }

    val v = opt[T1](fk)(toJsTyp)
    fk.value -> new TypedFieldOptions[js.Any] {
      val value               = v.asInstanceOf[js.Any]
      override val errors     = res
      override val touched    = false
      override val dirty      = true
      override val validating = false

    }

  }

  def err(name: String): (String, TypedFieldOptions[js.Any]) = {
    val res = {
      val list = t.l.filter(_.key == name).map(_.msg: js.Error | String)
      if (list.length == 0) js.undefined
      else js.defined(list)
    }

    name -> new TypedFieldOptions[js.Any] {
      val value               = null
      override val errors     = res
      override val touched    = false
      override val dirty      = true
      override val validating = false

    }
  }

  //========================= Step1Recommendation ======================
  def rec[T1](fk: FieldKey[T1])(implicit toJsTyp: JsCoerce[T1]): js.UndefOr[T1] = {
    dyn.step1_rec.selectDynamic(fk.value).asInstanceOf[UndefOr[js.Any]].flatMap { f =>
      toJsTyp.convert(f).orUndefined
    }
  }

  def recOpt[T1](fk: FieldKey[Option[T1]])(implicit toJsTyp: JsCoerce[T1]): js.UndefOr[T1] = {
    dyn.step1_rec.selectDynamic(fk.value).asInstanceOf[UndefOr[js.Any]].flatMap { f =>
      toJsTyp.convert(f).orUndefined
    }
  }

  def r: T = t
}
