package web.cps.claim

import ant.cps._
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import monix.eval.Task
import org.scalajs.dom
import relay.QueryRenderer
import relay.components._
import relay.generated.ClaimDetailView
import relay.graphql.TaggedNode
import relay.runtime.store.RelayEnvironment
import web.base.data.BaseViewerFrag
import web.routes.Routes.{ClaimsPage, Page}
import web.base.all._
import web.cps._
import web.cps.a1quote.FormCtl
import web.mutation.ClaimBuilderMutation
import web.react.RelayBackend
import web.routes.Routes
import web.views.Core
import shared.proto.TaggedClassSerDe._
import shared.proto.claims.CreateClaimReq

import scala.scalajs.js.JSConverters._
import scala.scalajs.js

object ClaimView {
  //
  case class Props(ctl: RouterCtl[Page], cp: ClaimsPage)
  val C = ScalaComponent
    .builder[Props]("ClaimsView")
    .stateless
    .renderBackend[Backend]
    .buildRelay[BaseViewerFrag]

  def list(msg: String) = {
    <.li(Icon(new Icon.Opt {
      val `type`: String = "check-circle-o"
    }.ex(^.marginRight := "10px", ^.color := "#f5a623")), msg)
  }

  class Backend(val $ : BackendScope[Props, Unit]) extends RelayBackend[BaseViewerFrag] {

    def startClaim =
      Callback((for {
        _ <- Task(dom.console.log("Starting Claim."))
        r <- ClaimBuilderMutation.start(relay.environment, CreateClaimReq(ec.currentTimeMillis()), relayProps.viewer.id)
        props <- $.props.toTask
        _ <- {
          if (r == null) Task.unit
          else {
            val res = r.get
            res match {
              case Left(value) => Task(dom.console.error(value.map(_.msg).mkString(", ")))
              case Right(value) if value.claimStart.claimEdge == null =>
                Task(dom.console.error(value.claimStart.error.get))
              case Right(value) =>
                props.ctl.set(ClaimsPage(value.claimStart.claimEdge.map(_.node.claimId).toOption)).toTask
            }
          }

        }
      } yield ()).runAsyncForget)

    def render(p: Props) = {
      Core.Container
        .rprops(relayProps)(Core.Props(false, p.ctl, Routes.ClaimsPage(None)))(
          <.div(
            <.div(^.className := bareLo + " " + bareLo("container"))(
              Card(new Card.Opts {
                override val className = "card-thck"
              })(FormCtl.arbSplitTTRow()((<.div("Click the button to get started..."), {
                show =>
                  Card(new Card.Opts {
                    override val bodyStyle  = (^.padding := "0px").toJs.styles
                    override val noHovering = true
                  }.ex(show.toMouseTagMod: _*))(BareCard.C(BareCard.Props(
                    "File a claim, we will walk you through the process and we'll get you" +
                      " paid as soon as possible.",
                    <.img(^.src := "/assets/images/07_auction_hammer_128.png"),
                    "File a New Claim"))(Button(new Button.Opt {
                    override val `type` = "primary"
                    override val size   = "large"
                  }, ^.width := "100%", ^.onClick --> startClaim)("I'd like to file a new claim")))
              }), (<.span(), { _ =>
                Card(new Card.Opts {
                  override val noHovering = true
                  override val title      = "Need Help?"
                })(
                  <.p("If you have questions we'd be happy to chat, click the link in the corner, and let's get" +
                    " started.  Or browse the articles below."),
                  <.ul(list("How to file a claim?"), list("Will I get in trouble if I file a claim?")))
              })))),
            ShortClaimsList.Container.rprops(relayProps)(p)))
    }
  }

  @gql("""
      fragment ClaimView_viewer on Viewer {
        id
        user {
          user {
            firstName
          }
        }
        claims(first: $first, after: $after) @connection(key: "ClaimView_claims") {
          edges {
            node {
              claimId status summary timeCreatedMs
            }
          }

        }
        ...Core_viewer
        ...ShortClaimsList_viewer
      }
    """)
  private[this] object fragList extends TaggedNode

  @gql("""
      query ClaimView($first: Int!, $after: String) {
        viewer {
          id
          ...ClaimView_viewer
        }
      }
    """)
  private[this] object query extends TaggedNode

  @gql("""
      query ClaimDetailView($id: ID!, $first: Int!) {
        viewer {
          id
          ...ClaimDetail_viewer
        }
        node(id: $id) {
          ... on Claim {
            ...ClaimDetail_claim
            details {
              ...ClaimDetail_details
            }
          }
        }
      }
    """)
  object queryDetail extends TaggedNode

  val Container = FragmentContainer.buildRelay(C, "viewer" -> ClaimDetailView.query)

  def QR(ctl: RouterCtl[Page], env: RelayEnvironment, page: ClaimsPage) = {
    val (vars, component, q) = page.id match {
      case Some(value) =>
        (new js.Object {
          val id: String = value
          val first      = 1000
        }, { (props: BaseViewerFrag) =>
          val dprops = js.Dynamic.literal(claim = props.node, details = props.node.asInstanceOf[js.Dynamic].details)
          val p      = props.asInstanceOf[ClaimDetail.RP].mergeLeftType(dprops)
          ClaimDetail.Container.rprops(p)(Props(ctl, page))
        }, queryDetail)
      case None =>
        (new js.Object {
          val id: String    = null
          val after: String = null
          val first         = 1000
        }, (props: BaseViewerFrag) => Container.rprops(props)(Props(ctl, page)), query)
    }

    QueryRenderer
      .builder[BaseViewerFrag](environment = env, query = q, variables = vars)
      .renderEither {
        case Some(Left(queryError)) =>
          val err = queryError.errors
          if (err.isDefined) {
            dom.console.error("Server issue", err.get)
            Alert(message = s"Sorry we encountered a problem")
          } else {
            dom.console.error("Unknown issue", queryError)
            <.div("Unknown issue.")
          }

        case Some(Right(props)) =>
          component(props)

        case None =>
          LoggedInSpinner.C()
      }
      .build
  }
}
