package web.cps.claim

import ant.cps.Table.ColumnProps
import ant.cps.{Badge, Card, Table}
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.vdom.html_<^._
import moment.Moment
import org.scalajs.dom
import relay.components._
import relay.generated.ShortClaimsList_viewer
import relay.graphql.TaggedNode
import web.base.data.{BaseViewerFrag, ClaimFrag, SangriaEdge}
import web.cps.bareLo
import web.react.RelayBackend
import web.routes.Routes.ClaimsPage

import scala.scalajs.js

object ShortClaimsList {

  @gql("""
       fragment ShortClaimsList_viewer on Viewer {
         claims(first: $first, after: $after) @connection(key: "ClaimView_claims") {
           edges {
             node {
               claimId status summary timeCreatedMs
             }
           }
           pageInfo { hasNextPage hasPreviousPage startCursor endCursor }
         }
       }
    """)
  object frag extends TaggedNode

  type RP    = BaseViewerFrag
  type Props = ClaimView.Props

  val C = ScalaComponent
    .builder[Props]("ShortClaimsList")
    .stateless
    .renderBackend[Backend]
    .buildRelay[RP]

  class Backend(val $ : BackendScope[Props, Unit]) extends RelayBackend[RP] {

    def numClaims =
      relayProps.viewer.claims.edges.length.toString + (if (relayProps.viewer.claims.pageInfo.hasNextPage)
                                                          "+"
                                                        else "")

    def render(p: Props) = {
      dom.console.log(relayProps)
      <.div(^.className := bareLo + " " + bareLo("container"))(Card(new Card.Opts {
        override val title     = <.span(<.span("You have "), <.strong(numClaims), <.span(" pending claims")).rawElement
        override val className = "card-thck"
      })(Table(new Table.Opts[SangriaEdge[ClaimFrag]] {
        val dataSource = relayProps.viewer.claims.edges
        override val pagination = new Table.PaginationConfig {
          override val pageSize = 20
        }
        override val rowKey = js.defined((r: SangriaEdge[ClaimFrag]) => r.node.claimId)

        val columns = js.Array[ColumnProps[SangriaEdge[ClaimFrag]]](
          Table.ColumnProps(title = "Id", dataIndex = "node.claimId", render = js.defined { (_, rec, _) =>
//                p.ctl.link(ClaimsPage(Some(rec.node.claimId)))
            p.ctl
              .link(ClaimsPage(Some(rec.node.claimId)))(rec.node.claimId)
              .rawElement
          }),
          Table.ColumnProps(title = "Summary", render = js.defined((_, rec, _) => rec.node.summary)),
          Table.ColumnProps(
            title = "Status",
            render = js.defined((_, rec, _) =>
              Badge
                .C(new Badge.Opts {
                  override val status =
                    js.defined(Badge.Status.processing)
                  override val text =
                    rec.node.status
                })()
                .rawElement)),
          Table.ColumnProps(
            title  = "Created",
            render = js.defined((_, rec, _) => Moment(rec.node.timeCreatedMs).fromNow())))
      })))
    }
  }

  val Container = FragmentContainer.buildRelay(C, "viewer" -> ShortClaimsList_viewer)
}
