package web.mutation

import dispalt.relay.gql
import com.trueaccord.scalapb.{GeneratedMessage, Message}
import org.scalajs.dom
import relay.graphql.TaggedNode
import relay.runtime.{RecordProxy, RecordSourceSelectorProxy}
import relay.runtime.RelayRuntime.{ConnectionHandler, ViewerHandler}
import relay.runtime.mutation.CommitMutation
import relay.runtime.store.RelayEnvironment
import shared.proto.TaggedClass
import web.base.data.{LocalRelayError, ProposedChangeFrag, SangriaEdge}
import web.base.all._

import scala.scalajs.js
import scala.scalajs.js.|

object ChangeMutation {

  @js.native
  trait ProposeChangeMutation extends js.Object {
    val proposedChange: SangriaEdge[ProposedChangeFrag] | Null
  }

  @js.native
  trait Result extends js.Object {
    val proposeChange: ProposeChangeMutation
  }

  @gql("""
        mutation ProposeChangeMutation(
          $input: ProposeChangeInput!
        ){
          proposeChange(input: $input) {
            clientMutationId
            proposedChange {
              __typename
              cursor
              node {
                id
                rawId
                when
                before
                after
              }
              cursor
            }
          }
        }
    """)
  object mut extends TaggedNode

  def commit[M <: GeneratedMessage with Message[M]: TaggedClass](env: RelayEnvironment, viewerId: String, msg: M) = {

    CommitMutation[PBufInput, Result](env, mut, PBufInput(msg), Some { rsp: RecordSourceSelectorProxy =>
      for {
        payload <- rsp.getRootField("proposeChange")
        viewer  <- rsp.get(viewerId)
//          viewer  <- rsp.get(ViewerHandler.VIEWER_ID)
        conn    <- ConnectionHandler.getConnection(viewer, "NameSettings_changes")
        newEdge <- payload.getLinkedRecord("proposedChange")
        // Edge can be null indicating no change.
        if newEdge != null
      } yield {
        ConnectionHandler.insertEdgeAfter(conn, newEdge)
      }; ()
    }).map(_.left.map(LocalRelayError(_)))
  }
}
